#include "archlib.h"

/* Print a system error message and exit */

void err_sys(char* message)
{
   perror(message);
   exit(EXIT_FAILURE);
}

/* Allocate memory, exit with an error-message, if no memory 
   available */ 

void* SecureMalloc(size_t size)
{
   void* res = malloc(size);

   if(!res)
   {
      printf("malloc() failure -- out of memory?");
      exit(EXIT_FAILURE);
   }
   return res;
 }

/* Same thing, just using realloc instead */

void *SecureRealloc(void* ptr, size_t size)
{
   void* res = realloc(ptr,size);

   if(!res)
   {
      printf("realloc() failure -- out of memory?");
      exit(EXIT_FAILURE);
   }
   return res;
 }


/* Print the character as a pair of two 4 bit nibbles, each shifted to
   positions 'A' to 'A'+15 */

void put_enc_char(unsigned char c, FILE *fp)
{
   int code1,code2;

   code1 = c % 16;
   code2 = c / 16;
   fputc('A'+code1, fp);
   fputc('A'+code2, fp);   
}

/* Print a string as a sequence of encoded characters. */

void put_enc_str(char *str, FILE* fp)
{
   while(*str)
   {
      put_enc_char(*str, fp);
      str++;
   }
}

/* Encode the file *infile and print result to *outfile */

void put_enc_file(FILE* infile, FILE* outfile)
{
   int c;

   while((c=fgetc(infile))!=EOF)
   {
      put_enc_char(c, outfile);
   }
}


void arch_file(char* filename)
{
   FILE *fp;
   
   fp = fopen(filename, "r");
   if(!fp)
   {
      err_sys("arch322 (open):");
   }
   while(*filename=='/')
   {
      fprintf(stderr, "Warning: Removed leading / from absolute pathname\n");
      filename++;
   }
   put_enc_str(filename, stdout);
   fputc('\n', stdout);
   put_enc_file(fp, stdout);
   fputc('\n', stdout);
   if(fclose(fp))
   {
      err_sys("arch322 (close):");	 
   }
}   


/* Check if a character is a possible encoded nibble */

void check_valid(int c)
{
   if(c < 'A' || c >= ('A'+16))
   {
      fprintf(stderr, "Illegal encoding detected\n");      
      exit(EXIT_FAILURE);
   }
}

/* Read an encoded char and return it. Return -1 on EOF, -2 if '\n' was
   read, print an errror if no valid character found */

int get_enc_char(FILE* fp)
{
   int c1, c2;

   c1 = fgetc(fp);
   if(c1 == EOF)
   {
      return -1;
   }
   if(c1 == '\n')
   {
      return -2;
   }
   check_valid(c1);
   c2 = fgetc(fp); 
   check_valid(c1);

   return (c2-'A')*16+(c1-'A');
}


/* Read an encoded string and return it, or NULL at EOF. String must
   be '\n' terminated. Caller must free memory! */

char* get_enc_str(FILE* fp)
{
   int c;
   int size = 2, i=0;
   char* buf = SecureMalloc(sizeof(char)*size);

   while((c=get_enc_char(fp))>=0)
   {
      buf[i] = c;
      i++;
      if(i==(size-1))
      {
	 size = size*2;
	 buf = SecureRealloc(buf,sizeof(char)*size);
      }
   }
   buf[i] = '\0';
   if(c==-1)
   {
      if(i)
      {
	 fprintf(stderr, "Incomplete String (corrupt archive)\n");
	 exit(EXIT_FAILURE);
      }
      free(buf);
      return NULL;
   }
   return buf;
}

/* Read an encoded, '\n'-terminated file from source and write it into
   filename */

void  dearch_file(char* filename, FILE* source)
{
   FILE* out;
   int c,i=0;

   while(*filename=='/')
   {
      fprintf(stderr, "Warning: Removed leading / from absolute pathname\n");
      filename++;
   }    
   fprintf(stderr,"Restoring: %s\n", filename);   
   out = fopen(filename, "w");
   if(!out)
   {
      err_sys("arch322 (open)");
   } 
   while((c=get_enc_char(source))>=0)
   {
      fputc(c, out);
      i++;
   }
   if(c==-1)
   {
      if(i)
      {
	 fprintf(stderr, "Incomplete File (corrupt archive)\n");
	 exit(EXIT_FAILURE);
      }
   }
   if(fclose(out))
   {
      err_sys("arch322 (close):");	 
   }
}








